'use strict';

/**
 * Stripe Payment Element script.
 * @version 1.0.0
 * @author OLD-ONE
 * @copyright OLD-ONE
 * @see stripe.elements(options?) https://stripe.com/docs/js/elements_object/create_without_intent
 * @see Card Element and Payment Element https://stripe.com/docs/payments/payment-card-element-comparison
 * @see elements.create("payment", options) https://stripe.com/docs/js/elements_object/create_payment_element
 * @see appearance https://stripe.com/docs/elements/appearance-api
 */
// Stripe public key
const stripePK = 'ここにStripeで取得した public key を設定';

// Server api url
const siteURL = location.href.replace(/^(.+)\/[^\/]*$/u, '$1');
const fetchURL = siteURL + '/payment_element.php';

// After payment message
const checkStatusMessage = {
  usd: {
    succeeded: 'Payment succeeded!',
    processing: 'Your payment is processing.',
    requires_payment_method: 'Your payment was not successful, please try again.',
    requires_capture: 'Payment verification succeeded!',
    requires_action: 'Your payment procedure succeeded!',
    default: 'Something went wrong.'
  },
  jpy: {
    succeeded: 'お支払いが完了しました。',
    processing: 'お支払いは処理中です。',
    requires_payment_method: 'お支払いは正常に完了しませんでした。もう一度お試しください。',
    requires_capture: 'お支払い認証が完了しました。',
    requires_action: 'お支払い手続きが完了しました。',
    default: '何か問題が発生しました。'
  }
};

// Form element
const form = document.getElementById('payment-form');

// Stripe order description
const description = 'Stripe order';

// Amount and currency
const amount = parseInt(form.amount.value, 10);
const currency = (form.currency) ? form.currency.value : 'jpy';

// Capture_method    Default is automatic.    [ automatic | automatic_async | manual ]
const capture_method = 'automatic'; // If this value is set to 'Manual', then 'Card' is the only payment method available.

// Set up future usage    [ null | off_session | on_session ]    https://stripe.com/docs/api/payment_intents/object#payment_intent_object-setup_future_usage
const setup_future_usage = null; // If this value is set to 'off_session' or 'on_session', then 'Card' is the only payment method available.

// URL where the customer should be redirected after the PaymentIntent is confirmed
const returnURL = location.href;
const addQueryAfterSubmit = true; // If this value is set to true, display checkStatusMessage after submit.
const isRedirect = false; // If this value is set to true, the checkStatusMessage will be displayed after submission, but the form input values will not be submitted.

// Stripe instance
const stripe = Stripe(stripePK);

// Peyment element appearance
const appearance = {
  Stripe: {theme: 'stripe'},
  Night: {theme: 'night', labels: 'floating'},
  Flat: {theme: 'flat'},
  Minimal: {
    theme: 'flat',
    variables: {
      fontFamily: ' "Gill Sans", sans-serif',
      fontLineHeight: '1.5',
      borderRadius: '10px',
      fontSizeBase: '16px',
      fontSizeSm: '14px',
      colorBackground: '#F6F8FA',
      accessibleColorOnColorPrimary: '#262626'
    },
    rules: {
      '.Block': {
        backgroundColor: 'var(--colorBackground)',
        boxShadow: 'none',
        padding: '12px'
      },
      '.Input': {
        padding: '12px'
      },
      '.Input:disabled, .Input--invalid:disabled': {
        color: 'lightgray'
      },
      '.Tab': {
        padding: '10px 12px 8px 12px',
        border: 'none'
      },
      '.Tab:hover': {
        border: 'none',
        boxShadow: '0px 1px 1px rgba(0, 0, 0, 0.03), 0px 3px 7px rgba(18, 42, 66, 0.04)'
      },
      '.Tab--selected, .Tab--selected:focus, .Tab--selected:hover': {
        border: 'none',
        backgroundColor: '#fff',
        boxShadow: '0 0 0 1.5px var(--colorPrimaryText), 0px 1px 1px rgba(0, 0, 0, 0.03), 0px 3px 7px rgba(18, 42, 66, 0.04)'
      },
      '.Label': {
        fontWeight: '500'
      }
    }
  },
  Bubblegum: {
    theme: 'stripe',
    variables: {
      fontWeightNormal: '500',
      fontSizeBase: '16px',
      fontSizeSm: '14px',
      borderRadius: '2px',
      colorPrimary: '#f360a6',
      tabIconSelectedColor: '#fff',
      gridRowSpacing: '16px'
    },
    rules: {
      '.Tab, .Input, .Block, .CheckboxInput, .CodeInput': {
        boxShadow: '0px 3px 10px rgba(18, 42, 66, 0.08)'
      },
      '.Block': {
        borderColor: 'transparent'
      },
      '.BlockDivider': {
        backgroundColor: '#ebebeb'
      },
      '.Tab, .Tab:hover, .Tab:focus': {
        border: '0'
      },
      '.Tab--selected, .Tab--selected:hover': {
        backgroundColor: '#f360a6',
        color: '#fff'
      }
    }
  },
  NinetyFive: {
    theme: 'flat',
    variables: {
      fontFamily: 'Verdana',
      fontLineHeight: '1.5',
      borderRadius: '0',
      fontSizeBase: '16px',
      fontSizeSm: '14px',
      colorBackground: '#fff',
      focusBoxShadow: 'none',
      focusOutline: '-webkit-focus-ring-color auto 1px',
      tabIconSelectedColor: 'var(--colorText)'
    },
    rules: {
      '.Input, .CheckboxInput, .CodeInput': {
        transition: 'none',
        boxShadow: 'inset -1px -1px #ffffff, inset 1px 1px #0a0a0a, inset -2px -2px #dfdfdf, inset 2px 2px #808080'
      },
      '.Input': {
        padding: '12px'
      },
      '.Input--invalid': {
        color: '#DF1B41'
      },
      '.Tab, .Block, .PickerItem--selected': {
        backgroundColor: '#dfdfdf',
        boxShadow: 'inset -1px -1px #0a0a0a, inset 1px 1px #ffffff, inset -2px -2px #808080, inset 2px 2px #dfdfdf'
      },
      '.Tab': {
        transition: 'none'
      },
      '.Tab:hover': {
        backgroundColor: '#eee'
      },
      '.Tab--selected, .Tab--selected:focus, .Tab--selected:hover': {
        color: 'var(--colorText)',
        backgroundColor: '#ccc'
      },
      '.Tab:focus, .Tab--selected:focus': {
        boxShadow: 'inset -1px -1px #0a0a0a, inset 1px 1px #ffffff, inset -2px -2px #808080, inset 2px 2px #dfdfdf',
        outline: 'none'
      },
      '.Tab:focus-visible': {
        outline: 'var(--focusOutline)'
      },
      '.PickerItem': {
        backgroundColor: '#dfdfdf',
        boxShadow: 'inset -1px -1px #0a0a0a, inset 1px 1px #ffffff, inset -2px -2px #808080, inset 2px 2px #dfdfdf',
        transition: 'none'
      },
      '.PickerItem:hover': {
        backgroundColor: '#eee'
      },
      '.PickerItem--highlight': {
        outline: '1px solid blue'
      },
      '.PickerItem--selected:hover': {
        backgroundColor: '#dfdfdf'
      }
    }
  },
  DarkBlue: {
    theme: 'night',
    variables: {
      fontFamily: 'Sohne, system-ui, sans-serif',
      fontWeightNormal: '500',
      borderRadius: '8px',
      colorBackground: '#0A2540',
      colorPrimary: '#EFC078',
      accessibleColorOnColorPrimary: '#1A1B25',
      colorText: 'white',
      colorTextSecondary: 'white',
      colorTextPlaceholder: '#727F96',
      tabIconColor: 'white',
      fontSizeBase: '16px',
      fontSizeSm: '14px',
      logoColor: 'dark'
    },
    rules: {
      '.Input, .Block': {
        backgroundColor: 'transparent',
        border: '1.5px solid var(--colorPrimary)'
      }
    }
  }
};

// Stripe elements options
const elementsOption = {
  locale: 'auto',
  mode: 'payment', // [ payment | setup | subscription ]
  captureMethod: capture_method,
  amount: amount,
  currency: currency,
//  payment_method_types: ['card', 'konbini'],
  setup_future_usage: setup_future_usage,
  appearance: appearance['Stripe']
};

// Stripe elements instance
const elements = stripe.elements(elementsOption);

// Stripe payment element options
const options = {
  layout: 'tabs', // [ accordion | tabs | object ]
  fields: {
    billingDetails: 'auto',
  },
  wallets: 'auto'
};

// Create payment element
const paymentElement = elements.create('payment', options);

// Add event listener after DOMContentLoaded
window.addEventListener('DOMContentLoaded', () => {
  paymentElement.mount('#payment-element');
  form.addEventListener('submit', (event) => {
    handleSubmit(event);
  });
  checkStatus();
});

// Error action
const handleError = (error) => {
  const errorMessage = error.message ?? 'An unexpected error occurred.';
  showMessage(errorMessage);
};

// Submit action
const handleSubmit = async (event) => {
  event.preventDefault();
  if (!stripe) {
    // Stripe.js hasn't yet loaded.
    // Make sure to disable form submission until Stripe.js has loaded.
    return;
  }
  setLoading(true);
  // Trigger form validation and wallet collection
  const {error} = await elements.submit();
  if (error) {
    handleError(error);
    setLoading(false);
    return;
  }
  // Create the SetupIntent and obtain clientSecret
  const data = {
    description: (form.description && form.description.value) ? form.description.value : description,
    amount: parseInt(form.amount.value, 10),
    currency: currency,
    capture_method: capture_method,
    setup_future_usage: setup_future_usage
  };
  data.city = (form.city && form.city.value) ? form.city.value : null;
  data.country = (form.country && form.country.value) ? form.country.value : null;
  data.line1 = (form.line1 && form.line1.value) ? form.line1.value : null;
  data.line2 = (form.line2 && form.line2.value) ? form.line2.value : null;
  data.postal_code = (form.postal_code && form.postal_code.value) ? form.postal_code.value : null;
  data.state = (form.state && form.state.value) ? form.state.value : null;
  data.email = (form.email && form.email.value) ? form.email.value : null;
  data.name = (form.name && form.name.value) ? form.name.value : null;
  data.phone = (form.phone && form.phone.value) ? form.phone.value : null;
  const address = (data.city || data.country || data.line1 || data.line2 || data.postal_code || data.state) ? {} : null;
  if (address !== null) {
    address.city = data.city;
    address.country = data.country;
    address.line1 = data.line1;
    address.line2 = data.line2;
    address.postal_code = data.postal_code;
    address.state = data.state;
  }
  const res = await fetch(fetchURL, {
    method: 'POST',
    headers: {'Content-Type': 'application/json'},
    body: JSON.stringify(data)
  });
  const {client_secret: clientSecret, payment_intent_id: paymentIntentId} = await res.json();
  const confirmPaymentArgs = {
    elements,
    clientSecret,
    confirmParams: {
      payment_method_data: {
        billing_details: {
          address: address,
          email: data.email,
          name: data.name,
          phone: data.phone
        },
      },
      // Return URL where the customer should be redirected after the PaymentIntent is confirmed.
      return_url: returnURL,
      receipt_email: data.email
    },
    // Uncomment below if you only want redirect for redirect-based payments
    redirect: 'if_required'
  };
  if (isRedirect) delete confirmPaymentArgs.redirect;
  stripe.confirmPayment(confirmPaymentArgs).then((result) => {
    if (result.error) {
      // Inform the customer that there was an error.
      handleError(result);
      setLoading(false);
    } else {
      stripePaymentIntentId(paymentIntentId, result.paymentIntent);
    }
  });
};

// Append payment intent id to form and submit form
const stripePaymentIntentId = (paymentIntentId, paymentIntent) => {
  const hiddenInput = document.createElement('input');
  hiddenInput.setAttribute('type', 'hidden');
  hiddenInput.setAttribute('name', 'payment_intent');
  hiddenInput.setAttribute('value', paymentIntentId);
  form.appendChild(hiddenInput);
  if (addQueryAfterSubmit) {
    const query = '?payment_intent=' + paymentIntentId + '&payment_intent_client_secret=' + paymentIntent.client_secret + '&redirect_status=' + paymentIntent.status;
    form.action = form.action.replace(/\?[^\?]*$/ui, '') + query;
  }
  form.submit();
};

// Show messages
const showMessage = (messageText, duration) => {
  duration = duration || 4000;
  const messageContainer = document.querySelector('#payment-message');
  messageContainer.classList.remove('hidden');
  messageContainer.textContent = messageText;
  setTimeout(() => {
    messageContainer.classList.add('hidden');
    messageContainer.textContent = '';
  }, duration);
};

// Show a spinner on payment submission
const setLoading = (isLoading) => {
  if (isLoading) {
    // Disable the button and show a spinner
    document.querySelector('#submitBtn').disabled = true;
    document.querySelector('#spinner').classList.remove('hidden');
    document.querySelector('#button-text').classList.add('hidden');
  } else {
    document.querySelector('#submitBtn').disabled = false;
    document.querySelector('#spinner').classList.add('hidden');
    document.querySelector('#button-text').classList.remove('hidden');
  }
}

// Fetches the payment intent status after payment submission
const checkStatus = async () => {
  const clientSecret = new URLSearchParams(window.location.search).get('payment_intent_client_secret');
  if (!clientSecret) {
    return;
  }
  const {paymentIntent} = await stripe.retrievePaymentIntent(clientSecret);
  const messageLang = checkStatusMessage[currency] || checkStatusMessage.en;
  const message = messageLang[paymentIntent.status] || messageLang.default;
  showMessage(message);
};
